/******************************************************************************
	File:			CCamera.cpp
	Description:		Implementation of the CCamera class
	Class:			CCamera
	Author:			AR1795 (10/14/05)							  
	Copyright (C) 2005 AR1795                                                                   
******************************************************************************/

#include "Camera.h"
#include "Timer.h"
#include "ADC.h"
#include <targets/LPC213x.h>        /* LPC21xx definitions */

static void adc0_isr (void) __attribute__ ((interrupt ("IRQ"))); 
static void timer1ISR(void) __attribute__ ((interrupt ("IRQ")));

WORD CCamera::s_wPixelCounter = 0;
CCamera::EState CCamera::s_eState = IDLE;    
BYTE CCamera::s_vbyPixels[CCamera::IMAGE_SIZE];

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CCamera::CCamera()
{       
  /* Timer 0 */
  /* Reset timer 0 */
  T0TCR = 0;
  /* Set Timer Mode: increment on every rising PCLK edge */
  T0CTCR = 0x00;
  /* Set the timer 0 prescale counter */
  T0PR = TIMER0_PRESCALER;
  /* Set timer 0 match register */
  T0MR0 = CAMERA_CLK_PERIOD;
  /* Reset counter on match */
  T0MCR = 2;
  /* Toggle the MAT0.0 External Match bit/output. */
  T0EMR = 3 << 4;
  /* Start timer 0 */
  T0TCR = 1;

  /* Timer 1 */
  /* Timer 1 interrupt is an IRQ interrupt */
  VICIntSelect &= ~(1 << TIMER1_VIC_CHANNEL);
  /* Enable timer 1 interrupt */
  VICIntEnable = 1 << TIMER1_VIC_CHANNEL;
  /* Use slot 1 for timer 0 interrupt */
  VICVectCntl1 = 0x20 | TIMER1_VIC_CHANNEL;
  /* Set the address of ISR for slot 1 */
  VICVectAddr1 = (unsigned int)timer1ISR;

  /* Set Counter Mode: TC is incremented on rising edges on the CAP1.2 input (1001)*/
  T1CTCR = 0x09;
  /* Set the timer 1 prescale counter */
  T1PR = TIMER1_PRESCALER;
  /* Reset timer 1 */
  T1TCR = 2;
  /* Generate interrupt and reset counter on match 0*/
  T1MCR = 3;
  /* Set timer 1 match register 0*/
  T1MR0 = DEFAULT_INTEGRATION_TIME;
  m_wIntegrationTime = DEFAULT_INTEGRATION_TIME;
  /* Set timer 1 match register 2*/
  T1MR2 = ST_PALSE_WIDTH;
  /* Set the MAT1.2 External Match bit/output on match */
  T1EMR = 2 << 8;
  /* Start timer 0 */
  T1TCR = 1;
  /* ADC */      
  /*
  Setup A/D: 10-bit AIN0 @ 3MHz 
  26:24 START = 010 
  27 EDGE = 1 
  Start conversion when the falling edge on P0.16/EINT0/MAT0.2/CAP0.2 pin.
  */
  AD0CR = 0x0B200401;
  VICIntSelect &= ~(1 << ADC_VIC_CHANNEL);
  VICIntEnClr  = 1 << ADC_VIC_CHANNEL;  /* Disable ADC interrupt */
  VICVectCntl0 = 0x20 | ADC_VIC_CHANNEL; 
  VICVectAddr0 = (unsigned long)adc0_isr;   // set interrupt vector in 0

  /* LED */
  IODIR0 = 0x80800000;  /* P0.31 and P0.23 defined as Output */
  IOSET0 = 0x80000000;  /* LED OFF */
  IOSET0 = 0x00800000;  /* Low gain */
  /* 
  PINSEL1 = 1100:
  3:2 = 11 => P0.17 = Match 1.2 (Timer 1)
  7:6 = 11 => P0.19 = Capture 1.2 (Timer 1)
  13:12 = 11 P0.22 => Match 0.0 (Timer 0)
  23:22 = 01 P0.27 => AD0.0
  19:18 = 10 P0.25 => Aout(DAC)
  */
  PINSEL1 = (3 << 2) | (3 << 6) | (3 << 12) | (1 << 22) | (2 << 18);
}

void CCamera::Start()
{
  if (s_eState == IDLE || s_eState == READY) 
  {                                                            
    /* Reset pixel counter */
    CCamera::s_wPixelCounter = 0;
    SetState (ARMED); 
  }
}


WORD CCamera::Read ()
{                                 
  WORD wPixel = 0xFFFF;
  if (s_wPixelCounter < IMAGE_SIZE)
  {
    wPixel = CCamera::s_vbyPixels[s_wPixelCounter++];
  }
  return wPixel;
}

void CCamera::Write (WORD a_wPixel)
{          
  if (s_wPixelCounter < IMAGE_SIZE)
  {                                       
    CCamera::s_vbyPixels[s_wPixelCounter++] = a_wPixel;
  }
  else
  {               
    /* Disable ADC interrupt */
    VICIntEnClr  = 1 << ADC_VIC_CHANNEL;  
    IOSET0 = 0x80000000;  /* OFF */

    SetState (READY);                     
    /* Reset pixel counter */
    CCamera::s_wPixelCounter = 0;         
  }
}

void CCamera::Reset()
{                                                                                
  if (s_eState == IDLE || s_eState == READY) 
  {
    /* Reset pixel counter */
    CCamera::s_wPixelCounter = 0;
  }
}

                         
WORD CCamera::SetIntgrationTime(WORD a_wTime)
{
  //Check integration time    
  a_wTime = (a_wTime > MIN_INTEGRATION_TIME)?a_wTime:MIN_INTEGRATION_TIME;
  a_wTime = (a_wTime < MAX_INTEGRATION_TIME)?a_wTime:MAX_INTEGRATION_TIME;    
  // save integration time
  m_wIntegrationTime = a_wTime;
  T1MR0 = m_wIntegrationTime;
  
  return a_wTime;
}

static void timer1ISR(void)
{
  /* Reset the MAT1.2 External Match bit/output */
  T1EMR &= ~0x00000004;
  /* Clear the timer 1 interrupt */
  T1IR = 0xFF;
  /* Update VIC priorities */
  VICVectAddr = 0;
  
  if( CCamera::ARMED == CCamera::GetState()) 
  {
    CCamera::SetState (CCamera::CAPTURE);  
    /* Enable ADC Interrupt */                       
    VICIntEnable = 1 << ADC_VIC_CHANNEL;   
    IOCLR0 = 0x80000000; /* TEST - LED ON */
  }
}


void adc0_isr (void) {
    unsigned long val = AD0DR;              /* Read A/D Data Register */
    AD0CR &= ~0x01000000;                   /* Stop A/D Conversion */
    VICVectAddr = 0;                       /* Acknowledge Interrupt */
    CCamera::Write (HIBYTE(val));
}
